/*
 * PishtiVisualiserApplet.java
 *
 * Created on 26 November 2006, 12:55
 */

package pishti;

import java.awt.Graphics;
import java.util.Collections;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;
import javax.swing.Icon;
import javax.swing.JLabel;
import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;
import javax.swing.text.StyledDocument;

/**
 *
 * @author  onur
 */
public class PishtiVisualiserApplet extends javax.swing.JApplet implements TableObserver {

    public static final boolean DEBUG = false;
    public static final boolean VISUALIZE = true;
    
    protected Vector players;
    private Player currentPlayer;
    private Table table;
    private Player lastCapturingPlayer;
    
    private Thread playThread;
    private Thread timerThread;
    private volatile boolean noStopRequested;
    private boolean paused;
    private final Object pauseLock = new Object();

    private int numCapturedP1, numCapturedP2, numCapturedP3, numCapturedP4;
    
    private Hashtable visualMappingOfPlayers;
    
    private StyledDocument doc; 
        
    public void initPlayers()
    {
        Player p1 = new ExamplePlayerImpl2(Player.PLAYER1, "Player1");
        Player p2 = new ExamplePlayerImpl2(Player.PLAYER2, "Player2");
        Player p3 = new ExamplePlayerImpl2(Player.PLAYER3, "Player3");
        Player p4 = new ExamplePlayerImpl(Player.PLAYER4, "Player4");
        
        players.add(p1);
        players.add(p2);
        players.add(p3);
        players.add(p4);
        
    }
    
    public void initGame()
    {
        Player p1 = (Player)players.elementAt(0);
        Player p2 = (Player)players.elementAt(1);
        Player p3 = (Player)players.elementAt(2);
        Player p4 = (Player)players.elementAt(3);
        
        VisualPlayer vp1 = new VisualPlayer();
        VisualPlayer vp2 = new VisualPlayer();
        VisualPlayer vp3 = new VisualPlayer();
        VisualPlayer vp4 = new VisualPlayer();
        
        JLabel[] cl1 = {jLabel$P1C1, jLabel$P1C2, jLabel$P1C3, jLabel$P1C4};
        JLabel[] cl2 = {jLabel$P2C1, jLabel$P2C2, jLabel$P2C3, jLabel$P2C4};
        JLabel[] cl3 = {jLabel$P3C1, jLabel$P3C2, jLabel$P3C3, jLabel$P3C4};
        JLabel[] cl4 = {jLabel$P4C1, jLabel$P4C2, jLabel$P4C3, jLabel$P4C4};
        
        vp1.setCardLabels(cl1);
        vp2.setCardLabels(cl2);
        vp3.setCardLabels(cl3);
        vp4.setCardLabels(cl4);
        
        visualMappingOfPlayers = new Hashtable();
        visualMappingOfPlayers.put(p1, vp1);
        visualMappingOfPlayers.put(p2, vp2);
        visualMappingOfPlayers.put(p3, vp3);
        visualMappingOfPlayers.put(p4, vp4);
        
        if(PishtiVisualiserApplet.VISUALIZE)
        {
            // player names
            jLabel1.setText(p1.getName());
            jLabel2.setText(p2.getName());
            jLabel3.setText(p3.getName());
            jLabel4.setText(p4.getName());
            jLabel5.setText(p1.getName());
            jLabel7.setText(p2.getName());
            jLabel9.setText(p3.getName());
            jLabel11.setText(p4.getName());
        }
        
        table.addObserver(p1);
        table.addObserver(p2);
        table.addObserver(p3);
        table.addObserver(p4);
        table.addObserver(this);
        
    }
    
    public void message(String s)
    {
        try {
            doc.insertString(doc.getLength(), "> " + s + "\n", doc.getStyle("regular"));
        } catch (BadLocationException ex) {
            ex.printStackTrace();
        }
    }
    
    public void play()
    {
        doc = jTextPane1.getStyledDocument();
        Style def = StyleContext.getDefaultStyleContext().
                        getStyle(StyleContext.DEFAULT_STYLE);

        Style regular = doc.addStyle("regular", def);
        StyleConstants.setFontFamily(def, "SansSerif");
        
        message("Started playing with a new pile");
        
        lastCapturingPlayer = null;
        numCapturedP1 = 0;
        numCapturedP2 = 0;
        numCapturedP3 = 0;
        numCapturedP4 = 0;
        
        Pile pile = new Pile();
        
        table.putInitialFourCards( pile.getTopFourCards() );
        
        while(!pile.isEmpty())
        {
            // deal 4 cards to each player
            for(Iterator it = players.iterator(); it.hasNext();)
            {
                Vector v = pile.getTopFourCards();
                Player p = (Player)it.next();
                
                if(PishtiVisualiserApplet.VISUALIZE)
                    ((VisualPlayer)visualMappingOfPlayers.get(p)).setHand(v);
                    //setHandIcons(p, v);
                
                p.setHand(v);
            }
            
            repaint();
            
            sleep(2000);
            
            for(int i=0; i<4; i++)
                for(Iterator it = players.iterator(); it.hasNext();)
                {
                        try {
                            waitWhilePaused();
        
                            currentPlayer = (Player)it.next();
                            Card cardToThrow = currentPlayer.throwCard();

                            if(PishtiVisualiserApplet.DEBUG) 
                                message("Player " + currentPlayer.getName() + " threw " + cardToThrow);
                                //System.out.println("Player " + currentPlayer.getName() + " threw " + cardToThrow);

                            table.put( cardToThrow );

                            if(PishtiVisualiserApplet.VISUALIZE)
                                ((VisualPlayer)visualMappingOfPlayers.get(currentPlayer)).removeCard(cardToThrow);

                            repaint();

                            sleep(2000);
                        } catch (InterruptedException ex) {
                            ex.printStackTrace();
                        }
                }
        }
        
        // give remaining cards to the last capturing player
        currentPlayer = lastCapturingPlayer;
        message("Remaining cards are given to " + currentPlayer.getName() + " because he/she was the last to capture the cards on the table");
        table.cleanTable();
        
        // give 3 more points to the majority.
        Hashtable ht = new Hashtable();
        ht.put(new Integer(numCapturedP1), "p1");
        ht.put(new Integer(numCapturedP2), "p2");
        ht.put(new Integer(numCapturedP3), "p3");
        ht.put(new Integer(numCapturedP4), "p4");
        Vector v = new Vector();
        v.add(new Integer(numCapturedP1));
        v.add(new Integer(numCapturedP2));
        v.add(new Integer(numCapturedP3));
        v.add(new Integer(numCapturedP4));

        //System.out.println("numCaptured: " + numCapturedP1 + " " + numCapturedP2 + " " + numCapturedP3 + " " + numCapturedP4);
        if(PishtiVisualiserApplet.DEBUG)
            message("numCaptured: " + numCapturedP1 + " " + numCapturedP2 + " " + numCapturedP3 + " " + numCapturedP4);
        
        Collections.sort(v);
        if(!v.elementAt(v.size()-1).equals(v.elementAt(v.size()-2))) // if two have the same num of cards, no majority points given
        { 
            String p = (String)ht.get(v.elementAt(v.size()-1));
            Player pl = null;
            
            if(p.equals("p1"))
            {
                //System.out.println("p1 got majority");
                pl = (Player)players.elementAt(0);
                pl.addPoints(3);
                jLabel6.setText(String.valueOf(pl.getPoints()));
            }
            else if(p.equals("p2"))
            {
                //System.out.println("p2 got majority");
                pl = (Player)players.elementAt(1);
                pl.addPoints(3);
                jLabel8.setText(String.valueOf(pl.getPoints()));
            }
            else if(p.equals("p3"))
            {
                //System.out.println("p3 got majority");
                pl = (Player)players.elementAt(2);
                pl.addPoints(3);
                jLabel10.setText(String.valueOf(pl.getPoints()));
            }
            else if(p.equals("p4"))
            {
                //System.out.println("p4 got majority");
                pl = (Player)players.elementAt(3);
                pl.addPoints(3);
                jLabel12.setText(String.valueOf(pl.getPoints()));
            }
            message(pl.getName() + " is given 3 points because he captured the highest number of cards");
        }
        jButton1.setEnabled(true);
        repaint();
    }

    public void notifyTableEvent(TableEvent evt)
    {
        if(evt instanceof TableCleanedEvent)
        {
            lastCapturingPlayer = currentPlayer;
            
            // increment number of captured cards
            switch(currentPlayer.getId())
            {
                case Player.PLAYER1:
                    numCapturedP1 += ((TableCleanedEvent)evt).getNumCardsOnTable();
                    break;
                case Player.PLAYER2:
                    numCapturedP2 += ((TableCleanedEvent)evt).getNumCardsOnTable();
                    break;
                case Player.PLAYER3:
                    numCapturedP3 += ((TableCleanedEvent)evt).getNumCardsOnTable();
                    break;
                case Player.PLAYER4:
                    numCapturedP4 += ((TableCleanedEvent)evt).getNumCardsOnTable();
                    break;
                default:
                    //System.out.println("Shouldn't have happened!");
                    break;
                    
            }
            
            // add points to currentPlayer
            currentPlayer.addPoints( ((TableCleanedEvent)evt).getPointsOnTable() );
            if(currentPlayer.getId() == Player.PLAYER1)
                jLabel6.setText(String.valueOf(currentPlayer.getPoints()));
            else if(currentPlayer.getId() == Player.PLAYER2)
                jLabel8.setText(String.valueOf(currentPlayer.getPoints()));
            else if(currentPlayer.getId() == Player.PLAYER3)
                jLabel10.setText(String.valueOf(currentPlayer.getPoints()));
            else if(currentPlayer.getId() == Player.PLAYER4)
                jLabel12.setText(String.valueOf(currentPlayer.getPoints()));
            
            //if(PishtiVisualiserApplet.DEBUG) 
                //System.out.println("Player " + currentPlayer.getName() + " got the table.");
            message(currentPlayer.getName() + " captured the cards on the table.");
            
            if(PishtiVisualiserApplet.VISUALIZE)
                jLabel$Table.setIcon(null);
        }
        else if(evt instanceof CardThrownEvent)
        {
            Card thrown = ((CardThrownEvent)evt).getThrown();
            
            if(PishtiVisualiserApplet.VISUALIZE)
                jLabel$Table.setIcon(getIconOfCard(thrown));
        }
    }
    
    public Icon getIconOfCard(Card c)
    {
        return new javax.swing.ImageIcon(getClass().getResource("/res/75/" + c.getTypeStr() + "-" + c.getValueStr() + "-75.png"));
    }
    
    private void sleep(int ms)
    {
        try {
                if(PishtiVisualiserApplet.VISUALIZE)
                    Thread.sleep(ms);
            } catch (InterruptedException ex) {
                ex.printStackTrace();
            }
    }
    
    /** Initializes the applet PishtiVisualiserApplet */
    public void init() 
    {
        players = new Vector();
        table = new Table();
        
        try {
            java.awt.EventQueue.invokeAndWait(new Runnable() {
                public void run() {
                    initComponents();
                    initPlayers();
                    initGame();
                }
            });
        } catch (Exception ex) {
            ex.printStackTrace();
        }        
    }
    
    private void startPlayThread()
    {
        paused = false;
        noStopRequested = true;
        
        Runnable r = new Runnable() {
            public void run() {
                play();
            }
        };
        playThread = new Thread(r, "Play");
        playThread.start();
    }
    
    private void stopPlayThread() 
    {
        noStopRequested = false;
        playThread.interrupt();
    }
    
    public void paint(Graphics g) 
    {
        getContentPane().paint(g);
    }

    public void destroy()
    {
        if(playThread != null)
            stopPlayThread();
    }
/*
    public void start() 
    {
        setPaused(false);
    }

    public void stop() 
    {
        setPaused(true);
    }
  */  

    
    /** This method is called from within the init() method to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        jPanel1 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        jLabel$P1C1 = new javax.swing.JLabel();
        jLabel$P1C2 = new javax.swing.JLabel();
        jLabel$P1C3 = new javax.swing.JLabel();
        jLabel$P1C4 = new javax.swing.JLabel();
        jLabel$P2C1 = new javax.swing.JLabel();
        jLabel$P2C2 = new javax.swing.JLabel();
        jLabel$P2C3 = new javax.swing.JLabel();
        jLabel$P2C4 = new javax.swing.JLabel();
        jLabel$P3C1 = new javax.swing.JLabel();
        jLabel$P3C2 = new javax.swing.JLabel();
        jLabel$P3C3 = new javax.swing.JLabel();
        jLabel$P3C4 = new javax.swing.JLabel();
        jLabel$P4C1 = new javax.swing.JLabel();
        jLabel$P4C2 = new javax.swing.JLabel();
        jLabel$P4C3 = new javax.swing.JLabel();
        jLabel$P4C4 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        jLabel$Table = new javax.swing.JLabel();
        jPanel2 = new javax.swing.JPanel();
        jButton1 = new javax.swing.JButton();
        jButton2 = new javax.swing.JButton();
        jLabel5 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        jLabel7 = new javax.swing.JLabel();
        jLabel8 = new javax.swing.JLabel();
        jLabel9 = new javax.swing.JLabel();
        jLabel10 = new javax.swing.JLabel();
        jLabel11 = new javax.swing.JLabel();
        jLabel12 = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        jTextPane1 = new javax.swing.JTextPane();

        getContentPane().setLayout(new java.awt.GridBagLayout());

        setForeground(new java.awt.Color(153, 0, 0));
        jPanel1.setLayout(new java.awt.GridBagLayout());

        jLabel1.setFont(new java.awt.Font("Dialog", 1, 18));
        jLabel1.setText("Player 1");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 5, 0);
        jPanel1.add(jLabel1, gridBagConstraints);

        jLabel$P1C1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/res/75/back-blue-75-1.png")));
        jLabel$P1C1.setDoubleBuffered(true);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 1;
        jPanel1.add(jLabel$P1C1, gridBagConstraints);

        jLabel$P1C2.setIcon(new javax.swing.ImageIcon(getClass().getResource("/res/75/back-blue-75-1.png")));
        jLabel$P1C2.setDoubleBuffered(true);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 1;
        jPanel1.add(jLabel$P1C2, gridBagConstraints);

        jLabel$P1C3.setIcon(new javax.swing.ImageIcon(getClass().getResource("/res/75/back-blue-75-1.png")));
        jLabel$P1C3.setDoubleBuffered(true);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 1;
        jPanel1.add(jLabel$P1C3, gridBagConstraints);

        jLabel$P1C4.setIcon(new javax.swing.ImageIcon(getClass().getResource("/res/75/back-blue-75-1.png")));
        jLabel$P1C4.setDoubleBuffered(true);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 5;
        gridBagConstraints.gridy = 1;
        jPanel1.add(jLabel$P1C4, gridBagConstraints);

        jLabel$P2C1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/res/75/back-blue-75-1.png")));
        jLabel$P2C1.setDoubleBuffered(true);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        jPanel1.add(jLabel$P2C1, gridBagConstraints);

        jLabel$P2C2.setIcon(new javax.swing.ImageIcon(getClass().getResource("/res/75/back-blue-75-1.png")));
        jLabel$P2C2.setDoubleBuffered(true);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        jPanel1.add(jLabel$P2C2, gridBagConstraints);

        jLabel$P2C3.setIcon(new javax.swing.ImageIcon(getClass().getResource("/res/75/back-blue-75-1.png")));
        jLabel$P2C3.setDoubleBuffered(true);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 4;
        jPanel1.add(jLabel$P2C3, gridBagConstraints);

        jLabel$P2C4.setIcon(new javax.swing.ImageIcon(getClass().getResource("/res/75/back-blue-75-1.png")));
        jLabel$P2C4.setDoubleBuffered(true);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 5;
        jPanel1.add(jLabel$P2C4, gridBagConstraints);

        jLabel$P3C1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/res/75/back-blue-75-1.png")));
        jLabel$P3C1.setDoubleBuffered(true);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 6;
        jPanel1.add(jLabel$P3C1, gridBagConstraints);

        jLabel$P3C2.setIcon(new javax.swing.ImageIcon(getClass().getResource("/res/75/back-blue-75-1.png")));
        jLabel$P3C2.setDoubleBuffered(true);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 6;
        jPanel1.add(jLabel$P3C2, gridBagConstraints);

        jLabel$P3C3.setIcon(new javax.swing.ImageIcon(getClass().getResource("/res/75/back-blue-75-1.png")));
        jLabel$P3C3.setDoubleBuffered(true);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 6;
        jPanel1.add(jLabel$P3C3, gridBagConstraints);

        jLabel$P3C4.setIcon(new javax.swing.ImageIcon(getClass().getResource("/res/75/back-blue-75-1.png")));
        jLabel$P3C4.setDoubleBuffered(true);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 5;
        gridBagConstraints.gridy = 6;
        jPanel1.add(jLabel$P3C4, gridBagConstraints);

        jLabel$P4C1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/res/75/back-blue-75-1.png")));
        jLabel$P4C1.setDoubleBuffered(true);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 6;
        gridBagConstraints.gridy = 2;
        jPanel1.add(jLabel$P4C1, gridBagConstraints);

        jLabel$P4C2.setIcon(new javax.swing.ImageIcon(getClass().getResource("/res/75/back-blue-75-1.png")));
        jLabel$P4C2.setDoubleBuffered(true);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 6;
        gridBagConstraints.gridy = 3;
        jPanel1.add(jLabel$P4C2, gridBagConstraints);

        jLabel$P4C3.setIcon(new javax.swing.ImageIcon(getClass().getResource("/res/75/back-blue-75-1.png")));
        jLabel$P4C3.setDoubleBuffered(true);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 6;
        gridBagConstraints.gridy = 4;
        jPanel1.add(jLabel$P4C3, gridBagConstraints);

        jLabel$P4C4.setIcon(new javax.swing.ImageIcon(getClass().getResource("/res/75/back-blue-75-1.png")));
        jLabel$P4C4.setDoubleBuffered(true);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 6;
        gridBagConstraints.gridy = 5;
        jPanel1.add(jLabel$P4C4, gridBagConstraints);

        jLabel2.setFont(new java.awt.Font("Dialog", 1, 18));
        jLabel2.setText("Player 2");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.gridheight = 2;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 0, 5);
        jPanel1.add(jLabel2, gridBagConstraints);

        jLabel3.setFont(new java.awt.Font("Dialog", 1, 18));
        jLabel3.setText("Player 3");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 7;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.insets = new java.awt.Insets(5, 0, 0, 0);
        jPanel1.add(jLabel3, gridBagConstraints);

        jLabel4.setFont(new java.awt.Font("Dialog", 1, 18));
        jLabel4.setText("Player 4");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 7;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.gridheight = 2;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 0);
        jPanel1.add(jLabel4, gridBagConstraints);

        jLabel$Table.setIcon(new javax.swing.ImageIcon(getClass().getResource("/res/75/back-blue-75-1.png")));
        jLabel$Table.setDoubleBuffered(true);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.gridheight = 2;
        jPanel1.add(jLabel$Table, gridBagConstraints);

        getContentPane().add(jPanel1, new java.awt.GridBagConstraints());

        jPanel2.setLayout(new java.awt.GridBagLayout());

        jButton1.setText("Play One Pile");
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        jPanel2.add(jButton1, gridBagConstraints);

        jButton2.setText("Pause");
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        jPanel2.add(jButton2, gridBagConstraints);

        jLabel5.setText("Player 1");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        jPanel2.add(jLabel5, gridBagConstraints);

        jLabel6.setForeground(new java.awt.Color(153, 0, 0));
        jLabel6.setText("0");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        jPanel2.add(jLabel6, gridBagConstraints);

        jLabel7.setText("Player 2");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        jPanel2.add(jLabel7, gridBagConstraints);

        jLabel8.setForeground(new java.awt.Color(153, 0, 0));
        jLabel8.setText("0");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        jPanel2.add(jLabel8, gridBagConstraints);

        jLabel9.setText("Player 3");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        jPanel2.add(jLabel9, gridBagConstraints);

        jLabel10.setForeground(new java.awt.Color(153, 0, 0));
        jLabel10.setText("0");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 7;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        jPanel2.add(jLabel10, gridBagConstraints);

        jLabel11.setText("Player 4");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 8;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        jPanel2.add(jLabel11, gridBagConstraints);

        jLabel12.setForeground(new java.awt.Color(153, 0, 0));
        jLabel12.setText("0");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 9;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        jPanel2.add(jLabel12, gridBagConstraints);

        jScrollPane1.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
        jScrollPane1.setAutoscrolls(true);
        jTextPane1.setEditable(false);
        jScrollPane1.setViewportView(jTextPane1);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 10;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanel2.add(jScrollPane1, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        getContentPane().add(jPanel2, gridBagConstraints);

    }// </editor-fold>//GEN-END:initComponents

    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
// TODO add your handling code here:
        if(evt.getActionCommand().equals("Pause"))
        {
            setPaused(true);
            jButton2.setText("Resume");
        }
        else
        {
            setPaused(false);
            jButton2.setText("Pause");
        }
    }//GEN-LAST:event_jButton2ActionPerformed

    private void setPaused(boolean newPauseState) {
    synchronized ( pauseLock ) {
      if ( paused != newPauseState ) {
        paused = newPauseState;
        pauseLock.notifyAll();
      }
    }
  }

  private void waitWhilePaused() throws InterruptedException {
    synchronized ( pauseLock ) {
      while ( paused ) {
        pauseLock.wait();
      }
    }
  }
    
    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
// TODO add your handling code here:
        startPlayThread();
        jButton1.setEnabled(false);
    }//GEN-LAST:event_jButton1ActionPerformed
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JLabel jLabel$P1C1;
    private javax.swing.JLabel jLabel$P1C2;
    private javax.swing.JLabel jLabel$P1C3;
    private javax.swing.JLabel jLabel$P1C4;
    private javax.swing.JLabel jLabel$P2C1;
    private javax.swing.JLabel jLabel$P2C2;
    private javax.swing.JLabel jLabel$P2C3;
    private javax.swing.JLabel jLabel$P2C4;
    private javax.swing.JLabel jLabel$P3C1;
    private javax.swing.JLabel jLabel$P3C2;
    private javax.swing.JLabel jLabel$P3C3;
    private javax.swing.JLabel jLabel$P3C4;
    private javax.swing.JLabel jLabel$P4C1;
    private javax.swing.JLabel jLabel$P4C2;
    private javax.swing.JLabel jLabel$P4C3;
    private javax.swing.JLabel jLabel$P4C4;
    private javax.swing.JLabel jLabel$Table;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel12;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTextPane jTextPane1;
    // End of variables declaration//GEN-END:variables
    
}

class VisualPlayer
{
    javax.swing.JLabel playerName;
    java.util.Hashtable hand;
    JLabel[] cardLabels;
    
    public VisualPlayer()
    {
        hand = new java.util.Hashtable();
    }
    
    public void setCardLabels(JLabel[] cardLabels)
    {
        this.cardLabels = cardLabels;
    }
    
    public void setHand(Vector v)
    {
        int i=0;
        for(Iterator it = v.iterator(); it.hasNext();)
        {
            Card c = (Card)it.next();
            hand.put(c, cardLabels[i]);
            cardLabels[i].setIcon( getIconOfCard(c) );
            i++;
        }
    }
    
    public void removeCard(Card c)
    {
        ((JLabel)hand.get(c)).setIcon( new javax.swing.ImageIcon(getClass().getResource("/res/75/back-blue-75-1.png")));
        hand.remove(c);
    }
 
    public Icon getIconOfCard(Card c)
    {
        return new javax.swing.ImageIcon(getClass().getResource("/res/75/" + c.getTypeStr() + "-" + c.getValueStr() + "-75.png"));
    }
    
}